@@ -4,8 +4,9 @@
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
+!     with contributions from 
 !     Christian Speckner <cnspeckn@googlemail.com>
-!     with contributions by Sebastian Schmidt
+!     Sebastian Schmidt
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by 
@@ -434,6 +435,14 @@
        status = 0
      end function gen_particle_status
 
+! extern "C" int gen_particle_is_beam( void* prt )
+     function gen_particle_is_beam (prt_obj) result (is_beam) bind(C)
+       use iso_c_binding
+       logical(c_bool) :: is_beam
+       type(c_ptr), value :: prt_obj
+       is_beam = .false.
+     end function gen_particle_is_beam
+
 ! extern "C" void* gen_particle_production_vertex( void* prt )
      type(c_ptr) function gen_particle_production_vertex (prt_obj) bind(C)
        use iso_c_binding
