@@ -1,4 +1,4 @@
-C $Id: mstw2008.f 2960 2011-01-16 17:08:22Z cnspeckn $
+C $Id: mstw2008.f 6124 2014-09-16 09:51:26Z kilian $
 
 C ---- FORTRAN 90 ----
 C
@@ -31,7 +31,7 @@ C----------------------------------------------------------------------
 C----------------------------------------------------------------------
 
       module mstw2008
-      use file_utils
+      use io_units
 
       contains
 
