@@ -1,4 +1,4 @@
-! WHIZARD 2.2.2 July 6 2014
+! WHIZARD 2.2.3 Nov 30 2014
 ! 
 ! Copyright (C) 1999-2014 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
@@ -6,8 +6,10 @@
 !     Juergen Reuter <juergen.reuter@desy.de>
 !     
 !     with contributions from
+!     Fabian Bach <fabian.bach@desy.de>
 !     Christian Speckner <cnspeckn@googlemail.com> 
-!     and  Fabian Bach, Felix Braam, Sebastian Schmidt, Daniel Wiesler 
+!     Christian Weiss <christian.weiss@desy.de>
+!     and Felix Braam, Sebastian Schmidt, Daniel Wiesler 
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by 
@@ -29,15 +31,15 @@
 
 module eio_raw
   
-  use kinds !NODEP!
-  use file_utils !NODEP!
-  use iso_varying_string, string_t => varying_string !NODEP!
-  use diagnostics !NODEP!
+  use kinds
+  use io_units
+  use iso_varying_string, string_t => varying_string
   use unit_tests
+  use diagnostics
 
-  use lorentz !NODEP!
+  use lorentz
   use variables
-  use models
+  use model_data
   use particles
   use beams
   use processes
@@ -79,7 +81,7 @@ contains
     class(eio_raw_t), intent(in) :: object
     integer, intent(in), optional :: unit
     integer :: u
-    u = output_unit (unit)
+    u = given_output_unit (unit)
     write (u, "(1x,A)")  "Raw event stream:"
     write (u, "(3x,A,L1)")  "Check MD5 sum     = ", object%check
     if (object%n_alt > 0) then
@@ -118,16 +120,22 @@ contains
     end if
   end subroutine eio_raw_set_parameters
     
-  subroutine eio_raw_init_out (eio, sample, process_ptr, data, success)
+  subroutine eio_raw_init_out &
+       (eio, sample, process_ptr, data, success, extension)
     class(eio_raw_t), intent(inout) :: eio
     type(string_t), intent(in) :: sample
+    type(string_t), intent(in), optional :: extension
     type(process_ptr_t), dimension(:), intent(in) :: process_ptr
     type(event_sample_data_t), intent(in), optional :: data
     logical, intent(out), optional :: success
     character(32) :: md5sum_prc, md5sum_cfg
     character(32), dimension(:), allocatable :: md5sum_alt
     integer :: i
-    eio%extension = "evx"
+    if (present (extension)) then
+       eio%extension  = extension
+    else
+       eio%extension = "evx"
+    end if
     eio%filename = sample // "." // eio%extension
     eio%unit = free_unit ()
     write (msg_buffer, "(A,A,A)")  "Events: writing to raw file '", &
@@ -350,7 +358,7 @@ contains
   
   subroutine eio_raw_1 (u)
     integer, intent(in) :: u
-    type(model_list_t) :: model_list
+    type(model_data_t), target :: model
     type(event_t), allocatable, target :: event
     type(process_t), allocatable, target :: process
     type(process_ptr_t) :: process_ptr
@@ -363,14 +371,14 @@ contains
     write (u, "(A)")  "*   Purpose: generate and read/write an event"
     write (u, "(A)")
 
-    call syntax_model_file_init ()
+    call model%init_test ()
 
     write (u, "(A)")  "* Initialize test process"
  
     allocate (process)
     process_ptr%ptr => process
     allocate (process_instance)
-    call prepare_test_process (process, process_instance, model_list)
+    call prepare_test_process (process, process_instance, model)
     call process_instance%setup_event_data ()
  
     allocate (event)
@@ -484,8 +492,7 @@ contains
     deallocate (process_instance)
     deallocate (process)
     
-    call model_list%final ()
-    call syntax_model_file_final ()
+    call model%final ()
 
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: eio_raw_1"
@@ -494,7 +501,7 @@ contains
   
   subroutine eio_raw_2 (u)
     integer, intent(in) :: u
-    type(model_list_t) :: model_list
+    type(model_data_t), target :: model
     type(var_list_t) :: var_list
     type(event_t), allocatable, target :: event
     type(process_t), allocatable, target :: process
@@ -510,14 +517,14 @@ contains
     write (u, "(A)")  "*            with multiple weights"
     write (u, "(A)")
 
-    call syntax_model_file_init ()
+    call model%init_test ()
 
     write (u, "(A)")  "* Initialize test process"
  
     allocate (process)
     process_ptr%ptr => process
     allocate (process_instance)
-    call prepare_test_process (process, process_instance, model_list)
+    call prepare_test_process (process, process_instance, model)
     call process_instance%setup_event_data ()
  
     call data%init (n_proc = 1, n_alt = 2)
@@ -598,8 +605,7 @@ contains
     deallocate (process_instance)
     deallocate (process)
     
-    call model_list%final ()
-    call syntax_model_file_final ()
+    call model%final ()
 
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: eio_raw_2"
