@@ -1,4 +1,4 @@
-! WHIZARD 2.2.2 July 6 2014
+! WHIZARD 2.2.3 Nov 30 2014
 ! 
 ! Copyright (C) 1999-2014 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
@@ -6,8 +6,10 @@
 !     Juergen Reuter <juergen.reuter@desy.de>
 !     
 !     with contributions from
+!     Fabian Bach <fabian.bach@desy.de>
 !     Christian Speckner <cnspeckn@googlemail.com> 
-!     and  Fabian Bach, Felix Braam, Sebastian Schmidt, Daniel Wiesler 
+!     Christian Weiss <christian.weiss@desy.de>
+!     and Felix Braam, Sebastian Schmidt, Daniel Wiesler 
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by 
@@ -29,14 +31,14 @@
 
 module eio_stdhep
   
-  use kinds !NODEP!
-  use file_utils !NODEP!
-  use iso_varying_string, string_t => varying_string !NODEP!
-  use diagnostics !NODEP!
+  use kinds
+  use io_units
+  use iso_varying_string, string_t => varying_string
   use unit_tests
+  use diagnostics
 
-  use lorentz !NODEP!
-  use models
+  use lorentz
+  use model_data
   use particles
   use beams
   use processes
@@ -112,7 +114,7 @@ contains
     class(eio_stdhep_t), intent(in) :: object
     integer, intent(in), optional :: unit
     integer :: u
-    u = output_unit (unit)
+    u = given_output_unit (unit)
     write (u, "(1x,A)")  "STDHEP event stream:"
     if (object%writing) then
        write (u, "(3x,A,A)")  "Writing to file   = ", char (object%filename)
@@ -151,13 +153,6 @@ contains
          call msg_fatal ("STDHEP: defined for scattering processes only")
     if (present (extension)) then
        eio%extension = extension
-    else
-       select type (eio)
-       type is (eio_stdhep_hepevt_t)
-          eio%extension = "hep"
-       type is (eio_stdhep_hepeup_t)
-          eio%extension = "up.hep"
-       end select
     end if
     call eio%set_filename ()    
     eio%unit = free_unit ()    
@@ -184,9 +179,11 @@ contains
     end if
   end subroutine eio_stdhep_split_out
   
-  subroutine eio_stdhep_init_out (eio, sample, process_ptr, data, success)
+  subroutine eio_stdhep_init_out &
+       (eio, sample, process_ptr, data, success, extension)
     class(eio_stdhep_t), intent(inout) :: eio
     type(string_t), intent(in) :: sample
+    type(string_t), intent(in), optional :: extension
     type(process_ptr_t), dimension(:), intent(in) :: process_ptr
     type(event_sample_data_t), intent(in), optional :: data
     logical, intent(out), optional :: success
@@ -195,7 +192,7 @@ contains
          call msg_bug ("STDHEP initialization: missing data")        
     eio%sample = sample
     call eio%set_splitting (data)    
-    call eio%common_init (sample, data)
+    call eio%common_init (sample, data, extension)
     eio%n_events_expected = data%n_evt
     write (msg_buffer, "(A,A,A)")  "Events: writing to STDHEP file '", &
          char (eio%filename), "'"
@@ -336,7 +333,7 @@ contains
   
   subroutine eio_stdhep_1 (u)
     integer, intent(in) :: u
-    type(model_list_t) :: model_list
+    type(model_data_t), target :: model
     type(event_t), allocatable, target :: event
     type(process_t), allocatable, target :: process
     type(process_ptr_t) :: process_ptr
@@ -352,14 +349,14 @@ contains
     write (u, "(A)")  "*      and write weight to file"
     write (u, "(A)")
 
-    call syntax_model_file_init ()
+    call model%init_test ()
 
     write (u, "(A)")  "* Initialize test process"
  
     allocate (process)
     process_ptr%ptr => process
     allocate (process_instance)
-    call prepare_test_process (process, process_instance, model_list)
+    call prepare_test_process (process, process_instance, model)
     call process_instance%setup_event_data ()
  
     allocate (event)
@@ -383,6 +380,10 @@ contains
     sample = "eio_stdhep_1"
  
     allocate (eio_stdhep_hepevt_t :: eio)
+    select type (eio)
+    type is (eio_stdhep_hepevt_t)
+       call eio%set_parameters ()
+    end select
     
     call eio%init_out (sample, [process_ptr], data)
     call event%generate (1, [0._default, 0._default])
@@ -433,8 +434,7 @@ contains
     deallocate (process_instance)
     deallocate (process)
 
-    call model_list%final ()
-    call syntax_model_file_final ()
+    call model%final ()
 
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: eio_stdhep_1"
@@ -443,7 +443,7 @@ contains
   
   subroutine eio_stdhep_2 (u)
     integer, intent(in) :: u
-    type(model_list_t) :: model_list
+    type(model_data_t), target :: model
     type(event_t), allocatable, target :: event
     type(process_t), allocatable, target :: process
     type(process_ptr_t) :: process_ptr
@@ -459,14 +459,14 @@ contains
     write (u, "(A)")  "*      and write weight to file"
     write (u, "(A)")
 
-    call syntax_model_file_init ()
+    call model%init_test ()
 
     write (u, "(A)")  "* Initialize test process"
  
     allocate (process)
     process_ptr%ptr => process
     allocate (process_instance)
-    call prepare_test_process (process, process_instance, model_list)
+    call prepare_test_process (process, process_instance, model)
     call process_instance%setup_event_data ()
  
     allocate (event)
@@ -490,6 +490,10 @@ contains
     sample = "eio_stdhep_2"
  
     allocate (eio_stdhep_hepeup_t :: eio)
+    select type (eio)
+    type is (eio_stdhep_hepeup_t)
+       call eio%set_parameters ()
+    end select    
     
     call eio%init_out (sample, [process_ptr], data)
     call event%generate (1, [0._default, 0._default])
@@ -540,8 +544,7 @@ contains
     deallocate (process_instance)
     deallocate (process)
 
-    call model_list%final ()
-    call syntax_model_file_final ()
+    call model%final ()
 
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: eio_stdhep_2"
