@@ -1,4 +1,4 @@
-! WHIZARD 2.2.2 July 6 2014
+! WHIZARD 2.2.3 Nov 30 2014
 ! 
 ! Copyright (C) 1999-2014 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
@@ -6,8 +6,10 @@
 !     Juergen Reuter <juergen.reuter@desy.de>
 !     
 !     with contributions from
+!     Fabian Bach <fabian.bach@desy.de>
 !     Christian Speckner <cnspeckn@googlemail.com> 
-!     and  Fabian Bach, Felix Braam, Sebastian Schmidt, Daniel Wiesler 
+!     Christian Weiss <christian.weiss@desy.de>
+!     and Felix Braam, Sebastian Schmidt, Daniel Wiesler 
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by 
@@ -29,18 +31,18 @@
 
 module sf_circe1
 
-  use kinds, only: default !NODEP!
-  use kinds, only: double !NODEP!
-  use iso_varying_string, string_t => varying_string !NODEP!
-  use file_utils !NODEP!
-  use limits, only: FMT_17, FMT_19 !NODEP!
-  use diagnostics !NODEP!
-  use os_interface
+  use kinds, only: default
+  use kinds, only: double
+  use iso_varying_string, string_t => varying_string
+  use io_units
+  use format_defs, only: FMT_17, FMT_19
   use unit_tests
-  use lorentz !NODEP!
+  use diagnostics
+  use physics_defs, only: UNDEFINED, ELECTRON, PHOTON
+  use lorentz
   use rng_base
   use pdg_arrays
-  use models
+  use model_data
   use flavors
   use colors
   use quantum_numbers
@@ -60,7 +62,7 @@ module sf_circe1
 
   type, extends (sf_data_t) :: circe1_data_t 
      private 
-     type(model_t), pointer :: model => null () 
+     class(model_data_t), pointer :: model => null () 
      type(flavor_t), dimension(2) :: flv_in 
      integer, dimension(2) :: pdg_in
      real(default), dimension(2) :: m_in = 0
@@ -116,7 +118,7 @@ contains
        (data, model, pdg_in, sqrts, eps, out_photon, &
         ver, rev, acc, chat)
     class(circe1_data_t), intent(out) :: data 
-    type(model_t), intent(in), target :: model
+    class(model_data_t), intent(in), target :: model
     type(pdg_array_t), dimension(2), intent(in) :: pdg_in
     real(default), intent(in) :: sqrts 
     real(default), intent(in) :: eps
@@ -174,7 +176,7 @@ contains
     integer, intent(in), optional :: unit 
     logical, intent(in), optional :: verbose
     integer :: u
-    u = output_unit (unit);  if (u < 0)  return 
+    u = given_output_unit (unit);  if (u < 0)  return 
     write (u, "(1x,A)") "CIRCE1 data:" 
     write (u, "(3x,A,2(1x,A))") "prt_in   =", &
          char (flavor_get_name (data%flv_in(1))), &
@@ -259,7 +261,7 @@ contains
     integer, intent(in), optional :: unit
     logical, intent(in), optional :: testflag
     integer :: u
-    u = output_unit (unit)
+    u = given_output_unit (unit)
     if (associated (object%data)) then
        call object%data%write (u)
        if (object%data%generate)  call object%rng_obj%rng%write (u)
@@ -508,9 +510,7 @@ contains
   
   subroutine sf_circe1_1 (u)
     integer, intent(in) :: u
-    type(os_data_t) :: os_data
-    type(model_list_t) :: model_list
-    type(model_t), pointer :: model
+    type(model_data_t), target :: model
     type(pdg_array_t), dimension(2) :: pdg_in
     type(pdg_array_t), dimension(2) :: pdg_out
     integer, dimension(:), allocatable :: pdg1, pdg2
@@ -524,10 +524,7 @@ contains
     write (u, "(A)")  "* Create empty data object"
     write (u, "(A)")
 
-    call os_data_init (os_data)
-    call syntax_model_file_init ()
-    call model_list%read_model (var_str ("QED"), &
-         var_str ("QED.mdl"), os_data, model)
+    call model%init_qed_test ()
     pdg_in(1) = ELECTRON
     pdg_in(2) = -ELECTRON
 
@@ -560,8 +557,7 @@ contains
     pdg2 = pdg_out(2)
     write (u, "(2x,99(1x,I0))")  pdg1, pdg2
 
-    call model_list%final ()
-    call syntax_model_file_final ()
+    call model%final ()
     
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_circe1_1"
@@ -570,9 +566,7 @@ contains
 
   subroutine sf_circe1_2 (u)
     integer, intent(in) :: u
-    type(os_data_t) :: os_data
-    type(model_list_t) :: model_list
-    type(model_t), pointer :: model
+    type(model_data_t), target :: model
     type(flavor_t), dimension(2) :: flv
     type(pdg_array_t), dimension(2) :: pdg_in
     class(sf_data_t), allocatable, target :: data
@@ -591,10 +585,7 @@ contains
     write (u, "(A)")  "* Initialize configuration data"
     write (u, "(A)")
 
-    call os_data_init (os_data)
-    call syntax_model_file_init ()
-    call model_list%read_model (var_str ("QED"), &
-         var_str ("QED.mdl"), os_data, model)
+    call model%init_qed_test ()
     call flavor_init (flv(1), ELECTRON, model)
     call flavor_init (flv(2), -ELECTRON, model)
     pdg_in(1) = ELECTRON
@@ -680,8 +671,7 @@ contains
     write (u, "(A)")  "* Cleanup"
 
     call sf_int%final ()
-    call model_list%final ()
-    call syntax_model_file_final ()
+    call model%final ()
     
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_circe1_2"
@@ -690,9 +680,7 @@ contains
 
   subroutine sf_circe1_3 (u)
     integer, intent(in) :: u
-    type(os_data_t) :: os_data
-    type(model_list_t) :: model_list
-    type(model_t), pointer :: model
+    type(model_data_t), target :: model
     type(flavor_t), dimension(2) :: flv
     type(pdg_array_t), dimension(2) :: pdg_in
     class(sf_data_t), allocatable, target :: data
@@ -711,10 +699,7 @@ contains
     write (u, "(A)")  "* Initialize configuration data"
     write (u, "(A)")
 
-    call os_data_init (os_data)
-    call syntax_model_file_init ()
-    call model_list%read_model (var_str ("QED"), &
-         var_str ("QED.mdl"), os_data, model)
+    call model%init_qed_test ()
     call flavor_init (flv(1), ELECTRON, model)
     call flavor_init (flv(2), -ELECTRON, model)
     pdg_in(1) = ELECTRON
@@ -786,8 +771,7 @@ contains
     write (u, "(A)")  "* Cleanup"
 
     call sf_int%final ()
-    call model_list%final ()
-    call syntax_model_file_final ()
+    call model%final ()
     
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_circe1_3"
