@@ -1,4 +1,4 @@
-! WHIZARD 2.2.2 July 6 2014
+! WHIZARD 2.2.3 Nov 30 2014
 ! 
 ! Copyright (C) 1999-2014 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
@@ -6,8 +6,10 @@
 !     Juergen Reuter <juergen.reuter@desy.de>
 !     
 !     with contributions from
+!     Fabian Bach <fabian.bach@desy.de>
 !     Christian Speckner <cnspeckn@googlemail.com> 
-!     and  Fabian Bach, Felix Braam, Sebastian Schmidt, Daniel Wiesler 
+!     Christian Weiss <christian.weiss@desy.de>
+!     and Felix Braam, Sebastian Schmidt, Daniel Wiesler 
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by 
@@ -29,19 +31,20 @@
 
 module sf_circe2
 
-  use kinds, only: default !NODEP!
-  use kinds, only: double !NODEP!
-  use iso_varying_string, string_t => varying_string !NODEP!
-  use file_utils !NODEP!
-  use limits, only: FMT_19 !NODEP!
-  use diagnostics !NODEP!
-  use os_interface
+  use kinds, only: default
+  use kinds, only: double
+  use iso_varying_string, string_t => varying_string
+  use io_units
+  use format_defs, only: FMT_19
   use unit_tests
-  use lorentz !NODEP!
+  use diagnostics
+  use os_interface
+  use physics_defs, only: UNDEFINED, PHOTON, ELECTRON
+  use lorentz
   use rng_base
   use selectors
   use pdg_arrays
-  use models
+  use model_data
   use flavors
   use helicities
   use quantum_numbers
@@ -60,7 +63,7 @@ module sf_circe2
 
   type, extends (sf_data_t) :: circe2_data_t
      private
-     type(model_t), pointer :: model => null () 
+     class(model_data_t), pointer :: model => null () 
      type(flavor_t), dimension(2) :: flv_in
      integer, dimension(2) :: pdg_in
      real(default) :: sqrts = 0
@@ -117,7 +120,7 @@ contains
        (data, os_data, model, pdg_in, sqrts, polarized, file, design)
     class(circe2_data_t), intent(out) :: data
     type(os_data_t), intent(in) :: os_data
-    type(model_t), intent(in), target :: model
+    class(model_data_t), intent(in), target :: model
     type(pdg_array_t), dimension(2), intent(in) :: pdg_in
     real(default), intent(in) :: sqrts
     logical, intent(in) :: polarized
@@ -211,7 +214,7 @@ contains
     integer, intent(in), optional :: unit
     logical, intent(in), optional :: verbose
     integer :: u, h
-    u = output_unit (unit)
+    u = given_output_unit (unit)
     write (u, "(1x,A)") "CIRCE2 data:"
     write (u, "(3x,A,A)")       "file   = ", char(data%filename)
     write (u, "(3x,A,A)")       "design = ", char(data%design)
@@ -282,7 +285,7 @@ contains
     integer, intent(in), optional :: unit
     logical, intent(in), optional :: testflag
     integer :: u
-    u = output_unit (unit)
+    u = given_output_unit (unit)
     if (associated (object%data)) then
        call object%data%write (u)
        call object%base_write (u, testflag)
@@ -439,8 +442,7 @@ contains
   subroutine sf_circe2_1 (u)
     integer, intent(in) :: u
     type(os_data_t) :: os_data
-    type(model_list_t) :: model_list
-    type(model_t), pointer :: model
+    type(model_data_t), target :: model
     type(pdg_array_t), dimension(2) :: pdg_in
     type(pdg_array_t), dimension(2) :: pdg_out
     integer, dimension(:), allocatable :: pdg1, pdg2
@@ -456,9 +458,7 @@ contains
     write (u, "(A)")
 
     call os_data_init (os_data)
-    call syntax_model_file_init ()
-    call model_list%read_model (var_str ("QED"), &
-         var_str ("QED.mdl"), os_data, model)
+    call model%init_qed_test ()
     pdg_in(1) = PHOTON
     pdg_in(2) = PHOTON
 
@@ -507,8 +507,7 @@ contains
 
     call data%write (u)
 
-    call model_list%final ()
-    call syntax_model_file_final ()
+    call model%final ()
     
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_circe2_1"
@@ -518,8 +517,7 @@ contains
   subroutine sf_circe2_2 (u)
     integer, intent(in) :: u
     type(os_data_t) :: os_data
-    type(model_list_t) :: model_list
-    type(model_t), pointer :: model
+    type(model_data_t), target :: model
     type(flavor_t), dimension(2) :: flv
     type(pdg_array_t), dimension(2) :: pdg_in
     class(sf_data_t), allocatable, target :: data
@@ -539,9 +537,7 @@ contains
     write (u, "(A)")
 
     call os_data_init (os_data)
-    call syntax_model_file_init ()
-    call model_list%read_model (var_str ("QED"), &
-         var_str ("QED.mdl"), os_data, model)
+    call model%init_qed_test ()
     call flavor_init (flv(1), PHOTON, model)
     call flavor_init (flv(2), PHOTON, model)
     pdg_in(1) = PHOTON
@@ -610,8 +606,7 @@ contains
     write (u, "(A)")  "* Cleanup"
 
     call sf_int%final ()
-    call model_list%final ()
-    call syntax_model_file_final ()
+    call model%final ()
     
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_circe2_2"
@@ -621,8 +616,7 @@ contains
   subroutine sf_circe2_3 (u)
     integer, intent(in) :: u
     type(os_data_t) :: os_data
-    type(model_list_t) :: model_list
-    type(model_t), pointer :: model
+    type(model_data_t), target :: model
     type(flavor_t), dimension(2) :: flv
     type(pdg_array_t), dimension(2) :: pdg_in
     class(sf_data_t), allocatable, target :: data
@@ -642,9 +636,7 @@ contains
     write (u, "(A)")
 
     call os_data_init (os_data)
-    call syntax_model_file_init ()
-    call model_list%read_model (var_str ("QED"), &
-         var_str ("QED.mdl"), os_data, model)
+    call model%init_qed_test ()
     call flavor_init (flv(1), PHOTON, model)
     call flavor_init (flv(2), PHOTON, model)
     pdg_in(1) = PHOTON
@@ -713,8 +705,7 @@ contains
     write (u, "(A)")  "* Cleanup"
 
     call sf_int%final ()
-    call model_list%final ()
-    call syntax_model_file_final ()
+    call model%final ()
     
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_circe2_3"
