@@ -1,4 +1,4 @@
-! WHIZARD 2.2.2 July 6 2014
+! WHIZARD 2.2.3 Nov 30 2014
 ! 
 ! Copyright (C) 1999-2014 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
@@ -6,8 +6,10 @@
 !     Juergen Reuter <juergen.reuter@desy.de>
 !     
 !     with contributions from
+!     Fabian Bach <fabian.bach@desy.de>
 !     Christian Speckner <cnspeckn@googlemail.com> 
-!     and  Fabian Bach, Felix Braam, Sebastian Schmidt, Daniel Wiesler 
+!     Christian Weiss <christian.weiss@desy.de>
+!     and Felix Braam, Sebastian Schmidt, Daniel Wiesler 
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by 
@@ -29,17 +31,17 @@
 
 module sf_epa
 
-  use kinds, only: default !NODEP!
-  use iso_varying_string, string_t => varying_string !NODEP!
-  use constants, only: pi !NODEP!
-  use file_utils !NODEP!
-  use limits, only: FMT_17, FMT_19 !NODEP!
-  use diagnostics !NODEP!
-  use lorentz !NODEP!
+  use kinds, only: default
+  use iso_varying_string, string_t => varying_string
+  use io_units
+  use constants, only: pi
+  use format_defs, only: FMT_17, FMT_19
   use unit_tests
-  use os_interface
+  use diagnostics
+  use physics_defs, only: ELECTRON, PHOTON
+  use lorentz
   use pdg_arrays
-  use models
+  use model_data
   use flavors
   use colors
   use quantum_numbers
@@ -64,7 +66,7 @@ module sf_epa
 
   type, extends(sf_data_t) :: epa_data_t
      private
-     type(model_t), pointer :: model => null ()
+     class(model_data_t), pointer :: model => null ()
      type(flavor_t), dimension(:), allocatable :: flv_in
      real(default) :: alpha
      real(default) :: x_min
@@ -112,7 +114,7 @@ contains
   subroutine epa_data_init &
        (data, model, pdg_in, alpha, x_min, q_min, E_max, mass, recoil)
     class(epa_data_t), intent(inout) :: data
-    type(model_t), intent(in), target :: model
+    class(model_data_t), intent(in), target :: model
     type(pdg_array_t), intent(in) :: pdg_in
     real(default), intent(in) :: alpha, x_min, q_min, E_max
     real(default), intent(in), optional :: mass
@@ -182,7 +184,7 @@ contains
     integer, intent(in), optional :: unit
     logical, intent(in), optional :: verbose
     integer :: u, i
-    u = output_unit (unit);  if (u < 0)  return
+    u = given_output_unit (unit);  if (u < 0)  return
     write (u, "(1x,A)") "EPA data:"
     if (allocated (data%flv_in)) then
        write (u, "(3x,A)", advance="no") "  flavor =  "
@@ -245,7 +247,7 @@ contains
     integer, intent(in), optional :: unit
     logical, intent(in), optional :: testflag
     integer :: u
-    u = output_unit (unit)
+    u = given_output_unit (unit)
     if (associated (object%data)) then
        call object%data%write (u)
        if (object%status >= SF_DONE_KINEMATICS) then
@@ -476,9 +478,7 @@ contains
   
   subroutine sf_epa_1 (u)
     integer, intent(in) :: u
-    type(os_data_t) :: os_data
-    type(model_list_t) :: model_list
-    type(model_t), pointer :: model
+    type(model_data_t), target :: model
     type(pdg_array_t) :: pdg_in
     type(pdg_array_t), dimension(1) :: pdg_out
     integer, dimension(:), allocatable :: pdg1
@@ -492,10 +492,7 @@ contains
     write (u, "(A)")  "* Create empty data object"
     write (u, "(A)")
 
-    call os_data_init (os_data)
-    call syntax_model_file_init ()
-    call model_list%read_model (var_str ("QED"), &
-         var_str ("QED.mdl"), os_data, model)
+    call model%init_qed_test ()
     pdg_in = ELECTRON
 
     allocate (epa_data_t :: data)
@@ -520,8 +517,7 @@ contains
     pdg1 = pdg_out(1)
     write (u, "(2x,99(1x,I0))")  pdg1
         
-    call model_list%final ()
-    call syntax_model_file_final ()
+    call model%final ()
     
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_epa_1"
@@ -530,9 +526,7 @@ contains
 
   subroutine sf_epa_2 (u)
     integer, intent(in) :: u
-    type(os_data_t) :: os_data
-    type(model_list_t) :: model_list
-    type(model_t), pointer :: model
+    type(model_data_t), target :: model
     type(flavor_t) :: flv
     type(pdg_array_t) :: pdg_in
     class(sf_data_t), allocatable, target :: data
@@ -551,10 +545,7 @@ contains
     write (u, "(A)")  "* Initialize configuration data"
     write (u, "(A)")
 
-    call os_data_init (os_data)
-    call syntax_model_file_init ()
-    call model_list%read_model (var_str ("QED"), &
-         var_str ("QED.mdl"), os_data, model)
+    call model%init_qed_test ()
     call flavor_init (flv, ELECTRON, model)
     pdg_in = ELECTRON
 
@@ -635,8 +626,7 @@ contains
     write (u, "(A)")  "* Cleanup"
     
     call sf_int%final ()
-    call model_list%final ()
-    call syntax_model_file_final ()
+    call model%final ()
     
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_epa_2"
@@ -645,9 +635,7 @@ contains
 
   subroutine sf_epa_3 (u)
     integer, intent(in) :: u
-    type(os_data_t) :: os_data
-    type(model_list_t) :: model_list
-    type(model_t), pointer :: model
+    type(model_data_t), target :: model
     type(flavor_t) :: flv
     type(pdg_array_t) :: pdg_in
     class(sf_data_t), allocatable, target :: data
@@ -666,10 +654,7 @@ contains
     write (u, "(A)")  "* Initialize configuration data"
     write (u, "(A)")
 
-    call os_data_init (os_data)
-    call syntax_model_file_init ()
-    call model_list%read_model (var_str ("QED"), &
-         var_str ("QED.mdl"), os_data, model)
+    call model%init_qed_test ()
     call flavor_init (flv, ELECTRON, model)
     pdg_in = ELECTRON
 
@@ -750,8 +735,7 @@ contains
     write (u, "(A)")  "* Cleanup"
     
     call sf_int%final ()
-    call model_list%final ()
-    call syntax_model_file_final ()
+    call model%final ()
     
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_epa_3"
@@ -760,9 +744,7 @@ contains
 
   subroutine sf_epa_4 (u)
     integer, intent(in) :: u
-    type(os_data_t) :: os_data
-    type(model_list_t) :: model_list
-    type(model_t), pointer :: model
+    type(model_data_t), target :: model
     type(flavor_t) :: flv
     type(pdg_array_t) :: pdg_in
     class(sf_data_t), allocatable, target :: data
@@ -781,10 +763,7 @@ contains
     write (u, "(A)")  "* Initialize configuration data"
     write (u, "(A)")
 
-    call os_data_init (os_data)
-    call syntax_model_file_init ()
-    call model_list%read_model (var_str ("QED"), &
-         var_str ("QED.mdl"), os_data, model)
+    call model%init_qed_test ()
     call flavor_init (flv, ELECTRON, model)
     pdg_in = ELECTRON
 
@@ -870,8 +849,7 @@ contains
     write (u, "(A)")  "* Cleanup"
     
     call sf_int%final ()
-    call model_list%final ()
-    call syntax_model_file_final ()
+    call model%final ()
     
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_epa_4"
@@ -880,9 +858,7 @@ contains
 
   subroutine sf_epa_5 (u)
     integer, intent(in) :: u
-    type(os_data_t) :: os_data
-    type(model_list_t) :: model_list
-    type(model_t), pointer :: model
+    type(model_data_t), target :: model
     type(flavor_t) :: flv
     type(pdg_array_t) :: pdg_in
     class(sf_data_t), allocatable, target :: data
@@ -900,10 +876,7 @@ contains
     write (u, "(A)")  "* Initialize configuration data"
     write (u, "(A)")
 
-    call os_data_init (os_data)
-    call syntax_model_file_init ()
-    call model_list%read_model (var_str ("SM"), &
-         var_str ("SM.mdl"), os_data, model)
+    call model%init_sm_test ()
     call flavor_init (flv, 1, model)
     pdg_in = [1, 2, -1, -2]
 
@@ -961,8 +934,7 @@ contains
     write (u, "(A)")  "* Cleanup"
     
     call sf_int%final ()
-    call model_list%final ()
-    call syntax_model_file_final ()
+    call model%final ()
     
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_epa_5"
